@@ -28,7 +28,6 @@
  * This should ultimately go in a header file.
  */
 void daemon_init(char *prog);
-int check_pid_valid(pid_t pid, char *prog);
 int check_process_running(char *prog, pid_t * pid);
 
 /*
@@ -38,7 +37,7 @@ static void update_pidfile(char *prog);
 static int setup_sigmask(void);
 
 
-int
+static int
 check_pid_valid(pid_t pid, char *prog)
 {
 	FILE *fp;
@@ -129,8 +128,13 @@ check_process_running(char *prog, pid_t * pid)
 	if (fp == NULL) {	/* error */
 		return 0;
 	}
-	fscanf(fp, "%d\n", &oldpid);
+
+	ret = fscanf(fp, "%d\n", &oldpid);
 	fclose(fp);
+
+	if ((ret == EOF) || (ret != 1))
+		return 0;
+
 	if (check_pid_valid(oldpid, cmd)) {
 		*pid = oldpid;
 		return 1;
@@ -213,7 +217,11 @@ daemon_init(char *prog)
 		exit(1);
 	}
 
-	daemon(0, 0);
+	if(daemon(0, 0)) {
+		fprintf(stderr, "daemon_init: Unable to daemonize.\n");
+		exit(1);
+	}
+
 
 	update_pidfile(prog);
 }
