@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**  
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
 #include "do_rack.h"
 
 char *pname = "fence_rack";
@@ -219,7 +207,7 @@ void get_options(int argc, char **argv)
 	    break;
 
 	  case 'V':
-	    printf("%s %s (built %s %s)\n", pname, FENCE_RELEASE_NAME,
+	    printf("%s %s (built %s %s)\n", pname, RELEASE_VERSION,
 		   __DATE__, __TIME__);
 	    printf("%s\n", REDHAT_COPYRIGHT);
 	    exit(DID_SUCCESS);
