@@ -2,7 +2,7 @@
 #include "ccs.h"
 #include "copyright.cf"
 
-#define OPTION_STRING			("cj:f:Dn:O:T:hVS")
+#define OPTION_STRING			("cj:f:Dn:O:hVSws")
 #define LOCKFILE_NAME			"/var/run/fenced.pid"
 
 struct client {
@@ -63,7 +63,7 @@ static int setup_ccs(fd_t *fd)
 	char path[256];
 	char *name = NULL, *str = NULL;
 	int error, cd, i = 0, count = 0;
-
+	int num_methods;
 
 	while ((cd = ccs_connect()) < 0) {
 		sleep(1);
@@ -104,6 +104,20 @@ static int setup_ccs(fd_t *fd)
 			free(str);
 	}
 
+	if (comline.skip_undefined_opt == FALSE) {
+		str = NULL;
+		memset(path, 0, 256);
+		sprintf(path, "/cluster/fence_daemon/@skip_undefined");
+
+		error = ccs_get(cd, path, &str);
+		if (!error)
+			comline.skip_undefined = atoi(str);
+		else
+			comline.skip_undefined = DEFAULT_SKIP_UNDEFINED;
+		if (str)
+			free(str);
+	}
+
 	if (comline.post_join_delay_opt == FALSE) {
 		str = NULL;
 		memset(path, 0, 256);
@@ -180,6 +194,14 @@ static int setup_ccs(fd_t *fd)
 		if (error || !name)
 			break;
 
+		num_methods = count_methods(cd, name);
+
+		if (comline.skip_undefined && !num_methods) {
+			log_debug("skip %s with zero methods", name);
+			free(name);
+			continue;
+		}
+
 		add_complete_node(fd, 0, name);
 		free(name);
 		count++;
@@ -519,6 +541,7 @@ static void print_usage(void)
 	printf("Options:\n");
 	printf("\n");
 	printf("  -c	       All nodes are in a clean state to start\n");
+	printf("  -s	       Skip startup fencing of nodes with no defined fence methods\n");
 	printf("  -j <secs>	Post-join fencing delay (default %d)\n",
 				   DEFAULT_POST_JOIN_DELAY);
 	printf("  -f <secs>	Post-fail fencing delay (default %d)\n",
@@ -590,6 +613,11 @@ static void decode_arguments(int argc, char **argv, commandline_t *comline)
 			comline->clean_start_opt = TRUE;
 			break;
 
+		case 's':
+			comline->skip_undefined = 1;
+			comline->skip_undefined_opt = TRUE;
+			break;
+
 		case 'j':
 			comline->post_join_delay = atoi(optarg);
 			comline->post_join_delay_opt = TRUE;
