@@ -1,17 +1,3 @@
-/*****************************************************************************
-******************************************************************************
-**
-**  gfs2_convert - convert a gfs1 filesystem into a gfs2 filesystem.
-**
-**  Copyright (C) 2006-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-******************************************************************************
-*****************************************************************************/
-
 #include <unistd.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -820,7 +806,7 @@ void give_warning(void)
 /* ------------------------------------------------------------------------- */
 void version(void)
 {
-	log_notice("gfs2_convert version %s (built %s %s)\n", GFS2_RELEASE_NAME,
+	log_notice("gfs2_convert version %s (built %s %s)\n", RELEASE_VERSION,
 			   __DATE__, __TIME__);
 	log_notice("%s\n\n", REDHAT_COPYRIGHT);
 }
@@ -1088,8 +1074,12 @@ int main(int argc, char **argv)
 	/* Make them seal their fate.                     */
 	/* ---------------------------------------------- */
 	if (!error) {
+		int abort;
+
 		give_warning();
-		if (!query(&opts, "Convert %s from GFS1 to GFS2? (y/n)", device)) {
+		if (!gfs2_query(&abort, &opts,
+				"Convert %s from GFS1 to GFS2? (y/n)",
+				device)) {
 			log_crit("%s not converted.\n", device);
 			close(sb2.device_fd);
 			exit(0);
