@@ -10,12 +10,12 @@
 #include "metawalk.h"
 
 static int remove_eattr_entry(struct gfs2_sbd *sdp,
-							  struct gfs2_buffer_head *leaf_bh,
-							  struct gfs2_ea_header *curr,
-							  struct gfs2_ea_header *prev)
+			      struct gfs2_buffer_head *leaf_bh,
+			      struct gfs2_ea_header *curr,
+			      struct gfs2_ea_header *prev)
 {
 	log_warn("Removing EA located in block #%"PRIu64" (0x%" PRIx64 ").\n",
-			 leaf_bh->b_blocknr, leaf_bh->b_blocknr);
+		 leaf_bh->b_blocknr, leaf_bh->b_blocknr);
 	if(!prev)
 		curr->ea_type = GFS2_EATYPE_UNUSED;
 	else {
@@ -30,25 +30,33 @@ static int remove_eattr_entry(struct gfs2_sbd *sdp,
 
 int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 		      uint64_t parent, struct gfs2_buffer_head **bh,
-		      void *private)
+		      enum update_flags *update, void *private)
 {
-	int *update = (int *) private;
 	struct gfs2_sbd *sbp = ip->i_sbd;
 	struct gfs2_block_query q;
 	struct gfs2_buffer_head *indir_bh = NULL;
 
+	*update = not_updated;
 	if(gfs2_check_range(sbp, block)) {
-		log_err("Extended attributes indirect block out of range...removing\n");
+		log_err("Extended attributes indirect block #%"PRIu64
+			" (0x%" PRIx64 ") for inode #%" PRIu64
+			" (0x%" PRIx64 ") out of range...removing\n",
+			block, block, ip->i_di.di_num.no_addr,
+			ip->i_di.di_num.no_addr);
 		ip->i_di.di_eattr = 0;
 		*update = (opts.no ? not_updated : updated);
 		return 1;
 	}
-	else if (gfs2_block_check(bl, block, &q)) {
+	else if (gfs2_block_check(sbp, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	else if(q.block_type != gfs2_indir_blk) {
-		log_err("Extended attributes indirect block invalid...removing\n");
+		log_err("Extended attributes indirect block #%"PRIu64
+			" (0x%" PRIx64 ") for inode #%" PRIu64
+			" (0x%" PRIx64 ") invalid...removing\n",
+			block, block, ip->i_di.di_num.no_addr,
+			ip->i_di.di_num.no_addr);
 		ip->i_di.di_eattr = 0;
 		*update = (opts.no ? not_updated : updated);
 		return 1;
@@ -59,25 +67,31 @@ int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 	*bh = indir_bh;
 	return 0;
 }
+
 int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
-		     uint64_t parent, struct gfs2_buffer_head **bh, void *private)
+		     uint64_t parent, struct gfs2_buffer_head **bh,
+		     enum update_flags *update, void *private)
 {
-	int *update = (int *) private;
 	struct gfs2_sbd *sbp = ip->i_sbd;
 	struct gfs2_block_query q;
 
+	*update = not_updated;
 	if(gfs2_check_range(sbp, block)) {
-		log_err("Extended attributes block out of range...removing\n");
+		log_err("Extended attributes block for inode #%" PRIu64
+			" (0x%" PRIx64 ") out of range...removing\n",
+			ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 		ip->i_di.di_eattr = 0;
 		*update = (opts.no ? not_updated : updated);
 		return 1;
 	}
-	else if (gfs2_block_check(bl, block, &q)) {
+	else if (gfs2_block_check(sbp, bl, block, &q)) {
 		stack;
 		return -1;
 	}
 	else if(q.block_type != gfs2_meta_eattr) {
-		log_err("Extended attributes block invalid...removing\n");
+		log_err("Extended attributes block for inode #%"PRIu64
+			" (0x%" PRIx64 ") invalid...removing\n",
+			ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 		ip->i_di.di_eattr = 0;
 		*update = (opts.no ? not_updated : updated);
 		return 1;
@@ -183,11 +197,13 @@ int check_eattr_extentry(struct gfs2_inode *ip, uint64_t *ea_ptr,
 			 struct gfs2_buffer_head *leaf_bh,
 			 struct gfs2_ea_header *ea_hdr,
 			 struct gfs2_ea_header *ea_hdr_prev,
-			 void *private)
+			 enum update_flags *want_updated, void *private)
 {
 	struct gfs2_block_query q;
 	struct gfs2_sbd *sbp = ip->i_sbd;
-	if(gfs2_block_check(bl, be64_to_cpu(*ea_ptr), &q)) {
+
+	*want_updated = not_updated;
+	if(gfs2_block_check(sbp, bl, be64_to_cpu(*ea_ptr), &q)) {
 		stack;
 		return -1;
 	}
@@ -208,18 +224,22 @@ int pass1c(struct gfs2_sbd *sbp)
 	uint64_t block_no = 0;
 	struct gfs2_buffer_head *bh;
 	struct gfs2_inode *ip = NULL;
-	int update = 0;
 	struct metawalk_fxns pass1c_fxns = { 0 };
 	int error = 0;
+	osi_list_t *tmp, *x;
+	struct special_blocks *ea_block;
+	enum update_flags want_updated = not_updated;
 
 	pass1c_fxns.check_eattr_indir = &check_eattr_indir;
 	pass1c_fxns.check_eattr_leaf = &check_eattr_leaf;
 	pass1c_fxns.check_eattr_entry = &check_eattr_entry;
 	pass1c_fxns.check_eattr_extentry = &check_eattr_extentry;
-	pass1c_fxns.private = (void *) &update;
+	pass1c_fxns.private = NULL;
 
 	log_info("Looking for inodes containing ea blocks...\n");
-	while (!gfs2_find_next_block_type(bl, gfs2_eattr_block, &block_no)) {
+	osi_list_foreach_safe(tmp, &sbp->eattr_blocks.list, x) {
+		ea_block = osi_list_entry(tmp, struct special_blocks, list);
+		block_no = ea_block->block;
 
 		if (skip_this_pass || fsck_abort) /* if asked to skip the rest */
 			return 0;
@@ -227,24 +247,25 @@ int pass1c(struct gfs2_sbd *sbp)
 		if (gfs2_check_meta(bh, GFS2_METATYPE_IN)) { /* if a dinode */
 			log_info("EA in inode %"PRIu64" (0x%" PRIx64 ")\n",
 				 block_no, block_no);
-			gfs2_block_clear(bl, block_no, gfs2_eattr_block);
+			gfs2_block_clear(sbp, bl, block_no, gfs2_eattr_block);
 			ip = fsck_inode_get(sbp, bh);
 
 			log_debug("Found eattr at %"PRIu64" (0x%" PRIx64 ")\n",
 				  ip->i_di.di_eattr, ip->i_di.di_eattr);
 			/* FIXME: Handle walking the eattr here */
-			error = check_inode_eattr(ip, &pass1c_fxns);
+			error = check_inode_eattr(ip, &want_updated,
+						  &pass1c_fxns);
 			if(error < 0) {
 				stack;
 				brelse(bh, not_updated);
 				return -1;
 			}
 
-			fsck_inode_put(ip, update); /* dinode_out, brelse, free */
+			fsck_inode_put(ip, want_updated); /* dinode_out,
+							     brelse, free */
 		} else {
-			brelse(bh, update);
+			brelse(bh, want_updated);
 		}
-		block_no++;
 	}
 	return 0;
 }
