@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -24,11 +12,11 @@
 #include <string.h>
 #include <errno.h>
 #include <inttypes.h>
-#include <magma.h>
 #include <netdb.h>
 #include <netinet/in.h>
-#include <linux/gnbd.h>
 
+#include "gnbd.h"
+#include "member_cman.h"
 #include "gnbd_endian.h"
 #include "list.h"
 #include "trans.h"
@@ -51,23 +39,39 @@ struct waiter_s {
 };
 typedef struct waiter_s waiter_t;
 
+struct down_node_s {
+  int nodeid;
+  list_t list;
+};
+typedef struct down_node_s down_node_t;
+
+#define MAX_NODES 256
+
+list_decl(down_node_list);
 list_decl(waiter_list);
 connection_t *connections;
 struct pollfd *polls;
 int max_id;
 char node_name[65];
 unsigned int checks = 0;
+cman_handle_t ch;
+cman_node_t nodes[MAX_NODES];
+int num_nodes;
+int cman_cb;
+int cman_reason;
 
 #define BUFSIZE (sizeof(monitor_info_t) + sizeof(uint32_t))
 #define RESTART_CHECK 10
 
-cluster_member_list_t *cluster_members;
 
 #define CLUSTER 0
 #define CONNECT 1
 
 list_t monitor_list;
 
+int can_shutdown(void *private){
+  return 0;
+}
 
 struct monitor_s {
   int minor_nr;
@@ -123,9 +127,12 @@ int monitor_device(int minor_nr, int timeout, char *server)
   return 0;
 }
 
+
 void setup_poll(void)
 {
   int i;
+
+  cman_cb = 0;
   polls = malloc(open_max() * sizeof(struct pollfd));
   if (!polls)
     fail_startup("cannot allocate poller structure : %s\n", strerror(errno));
@@ -133,19 +140,11 @@ void setup_poll(void)
   if (!connections)
     fail_startup("cannot allocate connection structures : %s\n",
                  strerror(errno));
-  polls[CLUSTER].fd = clu_connect(NULL, 0);
+  polls[CLUSTER].fd = setup_member(NULL);
   if (polls[CLUSTER].fd < 0)
-    fail_startup("cannot connect to cluster manager : %s\n",
-                 strerror(-(polls[CLUSTER].fd)));
-  cluster_members = clu_member_list(NULL);
-  if (!cluster_members)
-    fail_startup("cannot get initial member list\n");
-  if (memb_resolve_list(cluster_members, NULL) < 0)
-    fail_startup("cannot resolve member list\n");
+    fail_startup("cannot get cluster fd\n");
   polls[CLUSTER].events = POLLIN;
-  connections[CLUSTER].buf = malloc(BUFSIZE);
-  if (!connections[CLUSTER].buf)
-    fail_startup("couldn't allocation memory for cluster connection buffer\n");
+  connections[CLUSTER].buf = NULL;
   connections[CLUSTER].action = 0;
   connections[CLUSTER].size = 0;
   connections[CLUSTER].dev = -1;
@@ -251,13 +250,13 @@ int get_monitor_list(char **buffer, unsigned int *list_size)
   return 0;
 }
 
-cluster_member_t *check_for_node(cluster_member_list_t *list, char *node)
+cman_node_t *check_for_node(cman_node_t *list, int len, char *node)
 {
   int i;
 
-  for(i = 0; i < list->cml_count; i++){
-    if (!strcmp(list->cml_members[i].cm_name, node))
-      return &list->cml_members[i];
+  for(i = 0; i < len; i++){
+    if (!strcmp(list[i].cn_name, node))
+      return &list[i];
   }
   return NULL;
 }
@@ -336,6 +335,7 @@ void fail_device(monitor_t *dev)
   waiter_t *waiter;
 
   block_sigchld();
+  dev->state = FAILED_STATE;
   
   list_foreach_safe(list_item, &waiter_list, tmp) {
     waiter = list_entry(list_item, waiter_t, list);
@@ -355,41 +355,144 @@ void fail_device(monitor_t *dev)
   unblock_sigchld();
 }
 
-void fail_devices(cluster_member_list_t *nodes)
+static void fail_devices(char *node)
 {
   monitor_t *dev;
-  list_t *item, *next;
+  list_t *item;
 
-  if (!nodes)
-    return;
-  list_foreach_safe(item, &monitor_list, next){
+  list_foreach(item, &monitor_list) {
     dev = list_entry(item, monitor_t, list);
-    if (check_for_node(nodes, dev->server)){
+    if (strcmp(dev->server, node) == 0)
       fail_device(dev);
-      break;
+  }
+}
+
+static char *nodeid_to_name(int nodeid)
+{
+  int i;
+
+  for(i = 0; i < num_nodes; i++)
+    if (nodes[i].cn_nodeid == nodeid)
+      return nodes[i].cn_name;
+  log_err("cannot find node that matches nodeid %d\n", nodeid);
+  exit(1);
+}
+
+static void check_down_nodes(void)
+{
+  uint64_t fence_time;
+  int fenced;
+  down_node_t *node;
+  list_t *item, *next;
+  
+  list_foreach_safe(item, &down_node_list, next){
+    node = list_entry(item, down_node_t, list);
+    if (cman_get_fenceinfo(ch, node->nodeid, &fence_time, &fenced, NULL) < 0) {
+      log_err("cannot get fence info for nodeid %d : %s\n", node->nodeid,
+              strerror(errno));
+      exit(1);
+    }
+    if (fenced){
+      fail_devices(nodeid_to_name(node->nodeid));
+      list_del(&node->list);
+      free(node);
     }
   }
 }
 
-void handle_cluster_msg(void)
+static down_node_t *get_down_node(int nodeid)
 {
-  int event;
-  cluster_member_list_t *new, *lost;
+  list_t *item;
+  down_node_t *node;
 
-  event = clu_get_event(polls[CLUSTER].fd);
-  if (event == CE_SHUTDOWN){
-    log_err("lost connection to cluster manager\n");
-    /* FIXME -- need to retry.. Can't just give up */
+  list_foreach(item, &down_node_list) {
+    node = list_entry(item, down_node_t, list);
+    if (node->nodeid == nodeid)
+      return node;
+  }
+  return NULL;
+}
+
+static void get_initial_nodelist(void)
+{
+  if (cman_get_nodes(ch, MAX_NODES, &num_nodes, nodes) < 0) {
+    log_err("can't get initial cluster node list : %s\n", strerror(errno));
     exit(1);
   }
-  else if (event != CE_INQUORATE && event != CE_SUSPEND){
-    new = clu_member_list(NULL);
-    lost = memb_lost(cluster_members, new);
-    cml_free(cluster_members);
-    cluster_members = new;
+}
 
-    fail_devices(lost);
-    cml_free(lost);
+static void statechange(void)
+{
+  int ret, i;
+
+  num_nodes = 0;
+  memset(&nodes, 0, sizeof(nodes));
+  ret = cman_get_nodes(ch, MAX_NODES, &num_nodes, nodes);
+  if (ret < 0) {
+    log_err("can't get cluster node list : %s\n", strerror(errno));
+    exit(1);
+  }
+  for (i = 0; i < num_nodes; i++){
+    if (nodes[i].cn_member) {
+      down_node_t *node = get_down_node(nodes[i].cn_nodeid);
+      if (!node)
+        continue;
+      fail_devices(nodes[i].cn_name);
+      list_del(&node->list);
+      free(node);
+    }
+    else {
+      monitor_t *dev;
+      list_t *item;
+      if (get_down_node(nodes[i].cn_nodeid))
+        continue;
+      list_foreach(item, &monitor_list) {
+        down_node_t *node;
+        dev = list_entry(item, monitor_t, list);
+	if (strcmp(dev->server, nodes[i].cn_name) != 0)
+          continue;
+        if (dev->state == RESET_STATE || dev->state == RESTARTABLE_STATE ||
+            dev->state == FAILED_STATE)
+          continue;
+        node = malloc(sizeof(down_node_t));
+        if (!node) {
+          log_err("cannot allocate memory for down node %s\n",
+                  nodes[i].cn_name);
+          exit(1);
+        }
+        node->nodeid = nodes[i].cn_nodeid;
+        list_add(&node->list, &down_node_list);
+        break;
+      }
+    }
+  }
+}       
+
+void handle_cluster_msg(void)
+{
+  int ret;
+
+  while(1) {
+    ret = cman_dispatch(ch, CMAN_DISPATCH_ONE);
+    if (ret < 0)
+      break;
+
+    if (cman_cb) {
+      cman_cb = 0;
+      switch (cman_reason) {
+       case CMAN_REASON_STATECHANGE:
+        statechange();
+        break;
+       default:
+        break;
+      }
+    }
+    else
+      break;
+  }
+  if (ret == -1 && errno == EHOSTDOWN){
+    log_err("lost connection to cluster manager\n");
+    exit(1);
   }
 }
 
@@ -474,16 +577,15 @@ void handle_msg(int index){
   close_poller(index);
 }
 
-cluster_member_t *get_failover_server(monitor_t *dev)
+cman_node_t *get_failover_server(monitor_t *dev)
 {
-  char host[256];
-  cluster_member_t *server;
+  cman_node_t *server;
   list_t *item;
   monitor_t *other_dev;
 
-  server = check_for_node(cluster_members, dev->server);
+  server = check_for_node(nodes, num_nodes, dev->server);
   if (server == NULL){
-    log_err("server %s is not a cluster member, cannot fence.\n", host);
+    log_err("server %s is not a cluster member, cannot fence.\n", dev->server);
     return NULL;
   }
   list_foreach(item, &monitor_list){
@@ -575,7 +677,7 @@ int start_recvd(monitor_t *dev)
     exit(1);
   for(i = open_max()-1; i > 2; --i) 
     close(i);
-  execlp("gnbd_recvd", "gnbd_recvd", "-f", "-d", minor_str);
+  execlp("gnbd_recvd", "gnbd_recvd", "-f", "-d", minor_str, NULL);
   exit(1);
 }
 
@@ -619,15 +721,26 @@ void check_devices(void)
         dev->state = NORMAL_STATE;
       }
       else {
-        cluster_member_t *server;
+        cman_node_t *server;
         server = get_failover_server(dev);
         if (server){
-          if (clu_fence(server) < 0)
-            log_err("fence of %s failed\n", dev->server);
+          cman_handle_t ach;
+          ach = cman_admin_init(NULL);
+          if (!ach) {
+            log_err("cman_admin_init failure : %s\n", strerror(errno));
+            goto cant_fence;
+          }
+          if (cman_kill_node(ach, server->cn_nodeid) < 0){
+            log_err("fence of %s failed : %s\n", dev->server, strerror(errno));
+            cman_finish(ach);
+            goto cant_fence;
+          }
+          cman_finish(ach);
           dev->state = FENCED_STATE;
+          break;
         }
-        else
-          whack_recvd(dev);
+       cant_fence:
+        whack_recvd(dev);
       }
       break;
     case RESET_STATE:
@@ -641,6 +754,7 @@ void check_devices(void)
         start_recvd(dev);
       break;
     /* FENCED_STATE */
+    /* FAILED_STATE */
     }
   }
 }
@@ -673,6 +787,9 @@ void list_monitored_devs(void){
     case FENCED_STATE:
       strcpy(state, "fenced");
       break;
+    case FAILED_STATE:
+      strcpy(state, "failed");
+      break;
     }
     printf("%8d   %7d   %s\n", ptr->minor_nr, ptr->timeout, state);
   }
@@ -690,8 +807,11 @@ void do_poll(void)
       log_err("poll error : %s\n", strerror(errno));
     return;
   }
-  if (err == 0)
+  if (err == 0) {
     check_devices();
+    check_down_nodes();
+    return;
+  }
   for (i = 0; i <= max_id; i++){
     if (polls[i].revents & (POLLERR | POLLHUP | POLLNVAL)){
       log_err("Bad poll result, 0x%x on id %d\n", polls[i].revents, i);
@@ -773,14 +893,13 @@ int main(int argc, char *argv[])
   list_init(&monitor_list);
 
   setup_poll();
-
   err = monitor_device(minor_nr, timeout, argv[3]);
   if (err)
     fail_startup("cannot add device #%d to monitor_list : %s\n", minor_nr,
                  strerror(err));
   
   finish_startup("gnbd_monitor started. Monitoring device #%d\n", minor_nr);
-  
+  get_initial_nodelist();  
   while(1){
     do_poll();
   }
