@@ -5,6 +5,7 @@
 #include "gd_internal.h"
 
 static cman_handle_t	ch;
+static cman_handle_t	ch_admin;
 static int		old_quorate;
 static cman_node_t	old_nodes[MAX_NODES];
 static int		old_node_count;
@@ -12,19 +13,23 @@ static cman_node_t	cman_nodes[MAX_NODES];
 static int		cman_node_count;
 static char		name_buf[CMAN_MAX_NODENAME_LEN+1];
 
+int shutdown_cman(void)
+{
+	cman_leave_cluster(ch_admin, CMAN_LEAVEFLAG_FORCE);
+}
 
 int kill_cman(int nodeid)
 {
-	cman_handle_t ach;
+	return cman_kill_node(ch_admin, nodeid);
+}
+
+int set_cman_dirty(void)
+{
 	int rv;
 
-	ach = cman_admin_init(NULL);
-	if (!ach) {
-		log_print("cman_admin_init error %d %d", (int) ch, errno);
-		return -ENOTCONN;
-	}
-	rv = cman_kill_node(ach, nodeid);
-	cman_finish(ach);
+	rv = cman_set_dirty(ch_admin);
+	if (rv)
+		log_print("cman_set_dirty error %d", rv);
 	return rv;
 }
 
@@ -141,20 +146,24 @@ int setup_cman(void)
 		return -ENOTCONN;
 	}
 
+	ch_admin = cman_admin_init(NULL);
+	if (!ch_admin) {
+		log_print("cman_admin_init error %d %d", (int) ch_admin, errno);
+		rv = -ENOTCONN;
+		goto fail1;
+	}
+
 	rv = cman_start_notification(ch, cman_callback);
 	if (rv < 0) {
 		log_print("cman_start_notification error %d %d", rv, errno);
-		cman_finish(ch);
-		return rv;
+		goto fail2;
 	}
 
 	memset(&node, 0, sizeof(node));
 	rv = cman_get_node(ch, CMAN_NODEID_US, &node);
 	if (rv < 0) {
 		log_print("cman_get_node us error %d %d", rv, errno);
-		cman_stop_notification(ch);
-		cman_finish(ch);
-		goto out;
+		goto fail3;
 	}
 
 	cman_node_count = 0;
@@ -162,7 +171,7 @@ int setup_cman(void)
 	rv = cman_get_nodes(ch, MAX_NODES, &cman_node_count, cman_nodes);
 	if (rv < 0) {
 		log_print("cman_get_nodes error %d %d", rv, errno);
-		goto out;
+		goto fail3;
 	}
 
 	cman_quorate = cman_is_quorate(ch);
@@ -176,9 +185,14 @@ int setup_cman(void)
 
 	fd = cman_get_fd(ch);
 	client_add(fd, process_cman, close_cman);
+	return 0;
 
-	rv = 0;
- out:
+ fail3:
+	cman_stop_notification(ch);
+ fail2:
+	cman_finish(ch_admin);
+ fail1:
+	cman_finish(ch);
 	return rv;
 }
 
