@@ -39,6 +39,7 @@
 extern char *prog_name;
 extern int groupd_debug_opt;
 extern int groupd_debug_verbose;
+extern int groupd_shutdown_opt;
 extern char groupd_debug_buf[256];
 extern char dump_buf[DUMP_SIZE];
 extern int dump_point;
@@ -178,9 +179,10 @@ struct app {
 #define MSG_APP_RECOVER        3
 #define MSG_APP_INTERNAL       4
 #define MSG_GLOBAL_ID          5
+#define MSG_GROUP_VERSION      6
 
 #define MSG_VER_MAJOR          1
-#define MSG_VER_MINOR          0
+#define MSG_VER_MINOR          1
 #define MSG_VER_PATCH          0
 
 struct msg {
@@ -265,6 +267,7 @@ void client_dead(int ci);
 /* cman.c */
 int setup_cman(void);
 int kill_cman(int nodeid);
+int shutdown_cman(void);
 int set_cman_dirty(void);
 
 /* cpg.c */
