@@ -3,7 +3,7 @@
 
 #include "gd_internal.h"
 
-#define OPTION_STRING			"DhVv"
+#define OPTION_STRING			"Dhs:Vv"
 #define LOCKFILE_NAME			"/var/run/groupd.pid"
 #define LOG_FILE				"/var/log/groupd.log"
 
@@ -97,7 +97,7 @@ static int do_read(int fd, void *buf, size_t count)
 static int ignore_gfs_nolock(char *sysfs_dir, char *table)
 {
 	char path[PATH_MAX];
-	int rv, fd;
+	int fd;
 
 	if (!strstr(sysfs_dir, "gfs"))
 		return 0;
@@ -114,6 +114,7 @@ static int ignore_gfs_nolock(char *sysfs_dir, char *table)
 	if (fd < 0)
 		return 1;
 
+	close(fd);
 	return 0;
 }
 
@@ -146,6 +147,37 @@ static int kernel_instance_count(char *sysfs_dir)
 	return rv;
 }
 
+int check_dead_daemons(int ci)
+{
+	group_t *g;
+
+	if (strncmp(client[ci].type, "fence", 5) == 0) {
+		list_for_each_entry(g, &gd_groups, list) {
+			if (client[ci].level == g->level) {
+				return 1;
+			}
+		}
+	}
+
+	if (strncmp(client[ci].type, "dlm", 3) == 0) {
+		list_for_each_entry(g, &gd_groups, list) {
+			if (client[ci].level == g->level) {
+				return 1;
+			}
+		}
+	}
+
+	if (strncmp(client[ci].type, "gfs", 3) == 0) {
+		list_for_each_entry(g, &gd_groups, list) {
+			if (client[ci].level == g->level) {
+				return 1;
+			}
+		}
+	}
+
+	return 0;
+}
+
 int check_uncontrolled_groups(void)
 {
 	pid_t pid;
@@ -420,6 +452,11 @@ static void client_alloc(void)
 
 void client_dead(int ci)
 {
+	if (groupd_shutdown_opt && check_dead_daemons(ci)) {
+		log_print("%s daemon appears to be dead", client[ci].type);
+		shutdown_cman();
+	}
+
 	close(client[ci].fd);
 	client[ci].workfn = NULL;
 	client[ci].fd = -1;
@@ -869,6 +906,7 @@ static void print_usage(void)
 	printf("\n");
 	printf("  -D	       Enable debugging code and don't fork\n");
 	printf("  -h	       Print this help, then exit\n");
+	printf("  -s [0|1]     Enable (or disable) shutdown mode\n");
 	printf("  -V	       Print program version information, then exit\n");
 }
 
@@ -891,6 +929,10 @@ static void decode_arguments(int argc, char **argv)
 			exit(EXIT_SUCCESS);
 			break;
 
+		case 's':
+			groupd_shutdown_opt = atoi(optarg);
+			break;
+
 		case 'v':
 			groupd_debug_verbose++;
 			break;
@@ -1020,6 +1062,7 @@ void groupd_dump_save(void)
 char *prog_name;
 int groupd_debug_opt;
 int groupd_debug_verbose;
+int groupd_shutdown_opt = 1;
 char groupd_debug_buf[256];
 char dump_buf[DUMP_SIZE];
 int dump_point;
