@@ -131,8 +131,11 @@ check_process_running(char *prog, pid_t * pid)
 	if (fp == NULL) {	/* error */
 		return 0;
 	}
-	fscanf(fp, "%d\n", &oldpid);
+	ret = fscanf(fp, "%d\n", &oldpid);
 	fclose(fp);
+	if ((ret == EOF) || (ret != 1))
+		return 0;
+
 	if (check_pid_valid(oldpid, cmd)) {
 		*pid = oldpid;
 		return 1;
@@ -209,9 +212,14 @@ daemon_init(char *prog)
 		exit(1);
 	}
 
-	daemon(0, 0);
+	if (daemon(0, 0)) {
+		fprintf(stderr, "daemon_init: Unable to daemonize.\n");
+		exit(1);
+	}
 
 	update_pidfile(prog);
-	nice(-1);
+	if (nice(-1) < 0)
+		fprintf(stderr, "daemon_init: Unable to renice.\n");
+
 	//mlockall(MCL_CURRENT | MCL_FUTURE);
 }
