@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2002-2004
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 //#define DEBUG
 #include <assert.h>
 #include <platform.h>
@@ -481,6 +463,8 @@ get_rg_state_local(char *name, rg_state_t *svcblk)
  *                      3 = DO NOT stop service, return RG_EFORWARD
  *			4 = DO NOT stop service, return RG_EAGAIN
  *			5 = DO NOT stop service, return RG_EFROZEN
+ *			6 = DO NOT stop service, mark stopped and return
+ *			    RG_SUCCESS (0)
  */
 int
 svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
@@ -507,7 +491,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_STARTING:
 	case RG_STATE_RECOVER:
 	case RG_STATE_MIGRATE:
-		if ((svcStatus->rs_owner != my_id()) &&
+		if ((svcStatus->rs_owner != (uint32_t)my_id()) &&
 		    memb_online(membership, svcStatus->rs_owner)) {
 			/*
 			   Service is running and the owner is online.
@@ -533,7 +517,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		if (svcStatus->rs_owner == 0 ||
-		    (svcStatus->rs_owner == my_id())) {
+		    (svcStatus->rs_owner == (uint32_t)my_id())) {
 			/*
 			   Service is marked as running locally or on
 			   0 (e.g. no member).  Safe
@@ -545,9 +529,10 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 
 		/*
 		   Service is marked as running but node is down.
-		   Doesn't make much sense to stop it.
+		   Doesn't make much sense to stop it - but we need
+		   to mark it stopped
 		 */
-		ret = 2;
+		ret = 6;
 		break;
 
 	case RG_STATE_ERROR:
@@ -638,7 +623,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 	case RG_STATE_STARTED:
 	case RG_STATE_CHECK:
 	case RG_STATE_STARTING:
-		if (svcStatus->rs_owner == my_id()) {
+		if (svcStatus->rs_owner == (uint32_t)my_id()) {
 		    	/*
 			 * Service is already running locally
 			clulog(LOG_DEBUG,
@@ -648,7 +633,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 			break;
 		}
 
-		if (svcStatus->rs_owner != my_id() &&
+		if (svcStatus->rs_owner != (uint32_t)my_id() &&
 		    memb_online(membership, svcStatus->rs_owner)) {
 			/*
 			 * Service is running and the owner is online!
@@ -910,13 +895,13 @@ svc_migrate(char *svcName, int target)
 		return RG_EFAIL;
 	}
 
-	if (svcStatus.rs_owner == target) {
+	if (svcStatus.rs_owner == (uint32_t)target) {
 		rg_unlock(&lockp);
 		/* Do not allow migration to its current owner! */
 		return 0;
 	}
 
-	if (svcStatus.rs_owner != my_id()) {
+	if (svcStatus.rs_owner != (uint32_t)my_id()) {
 		rg_unlock(&lockp);
 		return RG_EFORWARD;
 	}
@@ -993,8 +978,8 @@ svc_migrate(char *svcName, int target)
 		return ret;
 	}
 
-	if (svcStatus.rs_last_owner != my_id() ||
-	    svcStatus.rs_owner != target ||
+	if (svcStatus.rs_last_owner != (uint32_t)my_id() ||
+	    svcStatus.rs_owner != (uint32_t)target ||
 	    svcStatus.rs_state != RG_STATE_MIGRATE) {
 		rg_unlock(&lockp);
 		return ret;
@@ -1166,7 +1151,7 @@ svc_status(char *svcName)
 		/* Don't check status if the service is frozen */
 		return 0;
 
-	if (svcStatus.rs_owner != my_id())
+	if (svcStatus.rs_owner != (uint32_t)my_id())
 		/* Don't check status for anything not owned */
 		return 0;
 
@@ -1186,7 +1171,8 @@ svc_status(char *svcName)
 
 
 static inline int
-handle_started_status(char *svcName, int ret, rg_state_t *svcStatus)
+handle_started_status(char *svcName, int ret,
+		      rg_state_t __attribute__((unused)) *svcStatus)
 {
 	int newowner;
 
@@ -1316,6 +1302,16 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 		clulog(LOG_DEBUG, "Unable to stop RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
 		return RG_EFAIL;
+	case 6:
+		/* Mark stopped, but do not do anything */
+		svcStatus.rs_last_owner = svcStatus.rs_owner;
+		svcStatus.rs_owner = 0;
+		svcStatus.rs_state = RG_STATE_STOPPED;
+		if (set_rg_state(svcName, &svcStatus) != 0) {
+			rg_unlock(&lockp);
+			return RG_EFAIL;
+		}
+		/* FALLTHROUGH */
 	case 2:
 		rg_unlock(&lockp);
 		return RG_ESUCCESS;
@@ -1476,7 +1472,7 @@ svc_fail(char *svcName)
 	}
 
 	if ((svcStatus.rs_state == RG_STATE_STARTED) &&
-	    (svcStatus.rs_owner != my_id())) {
+	    (svcStatus.rs_owner != (uint32_t)my_id())) {
 		rg_unlock(&lockp);
 		clulog(LOG_DEBUG, "Unable to disable RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
@@ -1524,7 +1520,8 @@ _svc_freeze(char *svcName, int enabled)
 		return RG_EFAIL;
 	}
 
-	clulog(LOG_DEBUG, "Handling %s request for RG %s\n", svcName, enabled?"freeze":"unfreeze");
+	clulog(LOG_DEBUG, "Handling %s request for RG %s\n", enabled?"freeze":"unfreeze",
+	       svcName);
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		rg_unlock(&lockp);
@@ -1555,7 +1552,7 @@ _svc_freeze(char *svcName, int enabled)
 
 	default:
 		rg_unlock(&lockp);
-		return RG_EFAIL;
+		return RG_EAGAIN;
 		break;
 	}
 
@@ -1613,7 +1610,7 @@ svc_start_remote(char *svcName, int request, uint32_t target)
 
 	/* Send relocate message to the other node */
 	if (msg_send(&ctx, &msg_relo, sizeof (SmMessageSt)) < 
-	    sizeof (SmMessageSt)) {
+	    (int)sizeof (SmMessageSt)) {
 		clulog(LOG_ERR,
 		       "#59: Error sending remote-start request to member #%d\n",
 		       target);
@@ -2060,8 +2057,8 @@ handle_start_remote_req(char *svcName, int req)
 
 	/* XXX ok, so we need to say "should I start this if I was the
 	   only cluster member online */
-	for (x = 0; x < membership->cml_count; x++) {
-		if (membership->cml_members[x].cn_nodeid == me)
+	for (x = 0; x < (int)membership->cml_count; x++) {
+		if (membership->cml_members[x].cn_nodeid == (int)me)
 			continue;
 
 		membership->cml_members[x].cn_member = 0;
